;*******************************************************************************
;                                                                              *
;   Filename:       Main.asm                    Projet: Ventillation 12F1822   *
;   Author:         Philippe DOUILLARD                                         *
;                                                                              *
;   Commentaire:    Module principal du projet                                 *
;                                                                              *
;   Version: 1.0    30/01/20142      Version Initiale du projet                *
;                                                                              *
;*******************************************************************************
;                                                                              *
;    Files required:    P12F1822.INC                                           *
;                                                                              *
;*********************** DIRECTIVE D'ASSEMBLAGE ********************************

    LIST P=12F1822              ; Directive to define processor and file format
    #include <P12F1822.INC>     ; Processor specific variable definitions

;************************* CONFIGURATION BITS **********************************

    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_OFF & _BORV_19 & _LVP_OFF

;********* VARIABLES General Purpose Register ********* 80 Bytes (20h-6Fh) *****

        CBLOCK 0x20
    ; Variable                  ; Variable
        ENDC

;******************** VARIABLES COMMON RAM ************ 16 Bytes (70h-7Fh) *****
        CBLOCK  0x70
    POTAR_LOW                ; Valeur ANA Low    POTENTIOMETRE
    POTAR_HIGH               ; Valeur ANA High   POTENTIOMETRE
    THERM_LOW                ; Valeur ANA Low    Thermoresistance
    THERM_HIGH               ; Valeur PWM High   Thermoresistance
    PWM_LOW                  ; Valeur PWM Low    Sortie Ventilateur
    PWM_HIGH                 ; Valeur PWM High   Sortie Ventilateur
    PWM_MIN_LOW              ; Valeur PWM Low    Limite Minimale
    PWM_MIN_HIGH             ; Valeur PWM High   Limite Minimale
    PWM_MAX_LOW              ; Valeur PWM Low    Limite Maximale
    PWM_MAX_HIGH             ; Valeur PWM High   Limite Maximale
    TEMP1                    ; Variable temporaire
    TEMP2                    ; Variable temporaire
    TEMP3                    ; Variable temporaire
        ENDC
;********************* DEFINITION DES CONSTANTE ********************************

;********************** EEPROM INITIALIZATION *****************256 Bytes *******

DATAEE    ORG  0xF000
    DE    "MCHP"  ; Place 'M' 'C' 'H' 'P' at address 0,1,2,3

;**************************** IN/OUT *******************************************

; PORTA,0	ICSPDAT             Rserv
; PORTA,1	ICSPCLK             Rserv
; PORTA,2	Analogue input      Potentiometre
; PORTA,3	MCLR                Rserv
; PORTA,4	Analogue input      Thermistance       -
; PORTA,5	PWM OUT             Ventilateur
;
;                                    12F1822
;                                   ----------
;                               Vdd |1      8| GND
;         Ventilateur (PWM) ->  RA5 |2      7| RA0 -> ICSPDAT
;        Thermistance (0-5V) -> RA4 |3      6| RA1 <- ICSPCLK
;                       MCLR -> RA3 |4      5| RA2 <- Potentiometre (0-5V)
;                                   ----------
;
;*******************************************************************************

;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------
    ORG     0x0000            ; processor reset vector
    GOTO    START             ; When using debug header, first inst.
;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------
    ORG      0x0004
;------------------------------------------------------------------------------
; USER INTERRUPT SERVICE ROUTINE GOES HERE
;------------------------------------------------------------------------------
    RETFIE                    ; return from interrupt
;*******************************************************************************
; MAIN PROGRAM
;*******************************************************************************
START
;--------------------------------------------------------- INIT I/O ------------
        BANKSEL PORTA           ; Selection Port A
        CLRF    PORTA           ; Init PORTA
        BANKSEL LATA            ; Selection Data Latch
        CLRF    LATA            ; Init LATA

        BANKSEL TRISA
        MOVLW   B'00011111'     ; Set RA5 as ouputs
        MOVWF   TRISA           ; and other all input

        BANKSEL ANSELA          ; Selection registre
        MOVLW   B'00010100'     ; Set RA2 RA4 as analogue inputs
        MOVWF   ANSELA          ; and set all other as digital i/o

        BANKSEL ADCON1          ;
        MOVLW   B'11110000'     ; Right justify, Frc clock
        MOVWF   ADCON1          ; Vdd and Vss Vref

        BANKSEL OSCCON  
        MOVLW   B'01111010'     ;PLL DISABLED, 16MHZ CLOCK SPEED, INTERNAL CLOCK
        MOVWF   OSCCON

        BANKSEL APFCON
        bsf     APFCON,0        ; Set PWM on RA5 (APFCON,0=1)

        BANKSEL PORTA
        CLRF    PORTA           ; CLEAR EVERYTHING ON PORT A

;--------------------------------------------------------- INIT PWM ------------
        BANKSEL TRISA
        bsf     TRISA,5         ; Disable PWM OUT

        BANKSEL CCP1CON
        MOVLW   B'00001100'     ; Select PWM mode for CCP module
        MOVWF   CCP1CON         ; Select single output on CCP1 pin (RA5)

        BANKSEL PR2
        MOVLW   0xff
        MOVWF   PR2             ; PWM period (PR2) = 255

        BANKSEL CCPR1L
        CLRF    CCPR1L          ; Clear high 8 bits of PWM duty cycle
        BANKSEL CCP1CON
        bcf     CCP1CON,4       ; clear low 2 bits of PWM Duty cycle DC1B
        bcf     CCP1CON,5       ; clear low 2 bits of PWM Duty cycle DC1B

        BANKSEL PIR1
        bcf     PIR1,1          ; Clear TMR2 interrupt flag
        BANKSEL T2CON
        MOVLW   b'01111011'
        ;CLRF    T2CON           ; Select TMR2 prescalar as divide by 1
        bsf     T2CON,2         ; turn TMR2 on (TMR2ON=1)

        BANKSEL PIR1
        BTFSs   PIR1,1          ; Interupt flag
        GOTO    $-1             ; No, test again

        BANKSEL TRISA
        bcf     TRISA,5         ; Enable PWM OUT
;----------------------------------------------------- INIT VARIABLES ---------
        CLRF    POTAR_LOW
        CLRF    POTAR_HIGH
        CLRF    THERM_LOW
        CLRF    THERM_HIGH
        CLRF    PWM_LOW
        CLRF    PWM_HIGH
        MOVLW   d'200'
        MOVWF   PWM_MIN_LOW
        MOVLW   d'01'
        MOVWF   PWM_MIN_HIGH
        MOVLW   b'11111111'
        MOVWF   PWM_MAX_LOW
        MOVLW   b'0000011'
        MOVWF   PWM_MAX_HIGH
        CLRF    TEMP1
        CLRF    TEMP2
        CLRF    TEMP3
;----------------------------------------------------- DEMARRAGE PROGRAME ------
        MOVLW   0xFF            ; PWM  100%
        BANKSEL CCPR1L
        MOVWF   CCPR1L          ; SET high 8 bits of PWM duty cycle
        BANKSEL CCP1CON
        bsf     CCP1CON,4       ; SET 2 bits of PWM Duty cycle DC1B
        bsf     CCP1CON,5       ; SET 2 bits of PWM Duty cycle DC1B

        MOVLW   d'60'           ; Nombre de cycle
        MOVWF   TEMP3           ; Charge compteur boucle
        CALL    LONG_DELAY      ; SUBROUTINE delay court
        decfsz  TEMP3           ; dcrmenter compteur boucle
        GOTO    $-2

        MOVF    PWM_MIN_LOW,W   ; Charge PWM avec Valeures minimales
        MOVWF   PWM_LOW
        MOVF    PWM_MIN_HIGH,W
        MOVWF   PWM_HIGH
;------------------------------------------------------- ANALOGIQUE POTAR ------
Boucle
        BANKSEL ADCON0
        MOVLW   B'00001001'     ; Select channel AN2
        MOVWF   ADCON0          ; Turn ADC On

        CALL    SHORT_DELAY     ; Acquisiton delay

        BSF     ADCON0,ADGO     ; Start conversion
        BTFSC   ADCON0,ADGO     ; Is conversion done?
        GOTO    $-1             ; No, test again

        BANKSEL ADRESH
        MOVF    ADRESH,W        ; Read upper 2 bits
        MOVWF   POTAR_HIGH      ; Store in variable High

        BANKSEL ADRESL          ;
        MOVF    ADRESL,W        ; Read lower 8 bits
        MOVWF   POTAR_LOW       ; Store in variable Low
;------------------------------------------------- ANALOGIQUE THERMORESITANCE --
        BANKSEL ADCON0
        MOVLW   B'00001101'     ; Select channel AN3
        MOVWF   ADCON0          ; Turn ADC On

        CALL    SHORT_DELAY     ; Acquisiton delay

        BSF     ADCON0,ADGO     ; Start conversion
        BTFSC   ADCON0,ADGO     ; Is conversion done?
        GOTO    $-1             ; No, test again

        BANKSEL ADRESH
        MOVF    ADRESH,W        ; Read upper 2 bits
        MOVWF   THERM_HIGH      ; Store in variable High

        BANKSEL ADRESL          ;
        MOVF    ADRESL,W        ; Read lower 8 bits
        MOVWF   THERM_LOW       ; Store in variable Low
;----------------------------------------------- COMPARAISON CONSIGNE/MESURE ---
        movf    POTAR_HIGH,w    ; Compare High
        subwf   THERM_HIGH,w    ; subtract Y-X MESURE-CONSIGNE (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    comp_res        ; Saut au resultat
        movf    POTAR_LOW,w     ; HIGH egal alors compar LOW
        subwf   THERM_LOW,w     ; subtract Y-X MESURE-CONSIGNE (Low)

comp_res
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    $+2
        GOTO    exit            ; = alors suite
        btfsc   STATUS,C        ; Skip if egual (X<Y alors C=0)
        GOTO    $+2
        GOTO    mesure_inf      ; < alors traitement Mesure<consigne
        btfss   STATUS,C        ; Skip if egual (X>Y alors C=1)
        GOTO    exit
        GOTO    mesure_sup      ; > alors traitement Mesure>consigne

;-------------------------------------------------------- MESURE < CONSIGNE ----
mesure_inf
        movf    PWM_MIN_HIGH,w  ; Compare High
        subwf   PWM_HIGH,w      ; subtract Y-X PWM-PWM_MIN High

        btfss   STATUS,Z        ; Skip if egual
        goto    mesure_res_inf  ; Saut au resultat
        movf    PWM_MIN_LOW,w   ; HIGH egal alors compar LOW
        subwf   PWM_LOW,w       ; subtract Y-X PWM-PWM_MIN Low

mesure_res_inf
        btfss   STATUS,Z        ; Skip if egual (X=Y alors Z=1)
        GOTO    $+2
        GOTO    exit            ; = alors suite
        btfss   STATUS,C        ; Skip if superior (X>Y alors C=1)
        GOTO    exit
        movf    PWM_LOW,F       ; Does LSB contain zero?
        btfsc   STATUS,Z        ; Skip if =0
        decf    PWM_HIGH,F      ; Not =0 then decrement the MSB
        decf    PWM_LOW,F       ; decrement the LSB
        GOTO    exit
;-------------------------------------------------------- MESURE > CONSIGNE ----
mesure_sup
        movf    PWM_MAX_HIGH,w  ; Compare High
        subwf   PWM_HIGH,w      ; subtract Y-X MIN-PWM (High)

        btfss   STATUS,Z        ; Skip if egual
        goto    mesure_res_sup
        movf    PWM_MAX_LOW,w   ; yes, they are equal then compare low
        subwf   PWM_LOW,w       ; subtract Y-X MIN-PWM-CONSIGNE Low

mesure_res_sup
        btfsc   STATUS,C        ; Skip if inferior (X<Y alors C=0)
        GOTO    exit
        incf    PWM_LOW,F       ; Add one to LSB
        btfsc   STATUS,Z        ; Overflowed?
        incf    PWM_HIGH,F      ; Yes, add one to MSB
        GOTO    exit
exit
;---------------------------------------------------------------- PWM OUT ------
        MOVF    PWM_LOW,W       ; Mise en forme PWM_LOW
        MOVWF   TEMP3
        LSRF    TEMP3,F
        LSRF    TEMP3,F

        btfsc   PWM_HIGH,0
        bsf     TEMP3,6         ; SET bits 6 of PWM_LOW
        btfss   PWM_HIGH,0
        bcf     TEMP3,6         ; RESET bits 6 of PWM_LOW

        btfsc   PWM_HIGH,1
        bsf     TEMP3,7         ; SET bits 7 of PWM_LOW
        btfss   PWM_HIGH,1
        bcf     TEMP3,7         ; RESET bits 7 of PWM_LOW
        MOVF    TEMP3,W
        BANKSEL CCPR1L
        MOVWF   CCPR1L          ; Charge PWM_LOW -> registre

        btfsc   PWM_LOW,0
        bsf     CCP1CON,4       ; SET bits 0 of PWM Duty cycle DC1B4
        btfss   PWM_LOW,0
        bcf     CCP1CON,4       ; RESET bits 0 of PWM Duty cycle DC1B4

        btfsc   PWM_LOW,1
        bsf     CCP1CON,5       ; SET bits 1 of PWM Duty cycle DC1B5
        btfss   PWM_LOW,1
        bcf     CCP1CON,5       ; RESET bits 1 of PWM Duty cycle DC1B5
;---------------------------------------------------------------- TEMPO CYCLE --
        CALL    LONG_DELAY
        BRA     Boucle
;*******************************************************************************
;                           SOUS ROUTINES
;*******************************************************************************

;--------------------------- SHORT_DELAY ---------------------------------------
SHORT_DELAY     movlw   d'20'       ; Nombre de cycle
                movwf   TEMP1       ; Charge compteur
                decfsz  TEMP1       ; dcrmenter compteur d'octets
                GOTO    $-1
                return
;---------------------------- LONG_DELAY ---------------------------------------
LONG_DELAY      movlw   d'255'      ; Nombre de cycle
                movwf   TEMP1       ; Charge compteur boucle1
                movlw   d'255'      ; Nombre de cycle
                movwf   TEMP2       ; Charge compteur boucle2

                decfsz  TEMP1       ; dcrmenter compteur boucle1
                GOTO    $-1         ; Saut arriere
                movlw   d'255'      ; Nombre de cycle
                movwf   TEMP1       ; Charge compteur boucle1
                decfsz  TEMP2       ; dcrmenter 2eme boucle
                GOTO    $-5
                return
;************************** END OF PROGRAM *************************************
                END

